@@ -4,10 +4,12 @@
  */
 
 #include "rdpsession.h"
+#include "rdpcliprdr.h"
 #include "rdphostpreferences.h"
 
 #include <memory>
 
+#include <QApplication>
 #include <QKeyEvent>
 #include <QMouseEvent>
 
@@ -20,7 +22,6 @@
 #include <freerdp/channels/rdpgfx.h>
 #include <freerdp/client.h>
 #include <freerdp/client/channels.h>
-#include <freerdp/client/cliprdr.h>
 #include <freerdp/client/cmdline.h>
 #include <freerdp/client/rdpgfx.h>
 #include <freerdp/event.h>
@@ -183,25 +184,37 @@ BOOL resizeDisplay(rdpContext *context)
     return FALSE;
 }
 
+BOOL playSound(rdpContext *context, const PLAY_SOUND_UPDATE *play_sound)
+{
+    Q_UNUSED(play_sound);
+    auto session = reinterpret_cast<RdpContext *>(context)->session;
+    if (session->onPlaySound()) {
+        return TRUE;
+    }
+    return FALSE;
+}
+
 void channelConnected(void *context, ChannelConnectedEventArgs *e)
 {
-    auto rdpC = reinterpret_cast<rdpContext *>(context);
     if (strcmp(e->name, RDPGFX_DVC_CHANNEL_NAME) == 0) {
+        rdpContext *rdpC = reinterpret_cast<rdpContext *>(context);
         gdi_graphics_pipeline_init(rdpC->gdi, (RdpgfxClientContext *)e->pInterface);
     } else if (strcmp(e->name, CLIPRDR_SVC_CHANNEL_NAME) == 0) {
-        CliprdrClientContext *clip = (CliprdrClientContext *)e->pInterface;
-        clip->custom = context;
+        auto session = reinterpret_cast<RdpContext *>(context)->session;
+        auto krdcContext = reinterpret_cast<RdpContext *>(context);
+        auto cliprdr = reinterpret_cast<CliprdrClientContext *>(e->pInterface);
+        session->initializeClipboard(krdcContext, cliprdr);
     }
 }
 
 void channelDisconnected(void *context, ChannelDisconnectedEventArgs *e)
 {
-    auto rdpC = reinterpret_cast<rdpContext *>(context);
     if (strcmp(e->name, RDPGFX_DVC_CHANNEL_NAME) == 0) {
+        rdpContext *rdpC = reinterpret_cast<rdpContext *>(context);
         gdi_graphics_pipeline_uninit(rdpC->gdi, (RdpgfxClientContext *)e->pInterface);
     } else if (strcmp(e->name, CLIPRDR_SVC_CHANNEL_NAME) == 0) {
-        CliprdrClientContext *clip = (CliprdrClientContext *)e->pInterface;
-        clip->custom = nullptr;
+        auto session = reinterpret_cast<RdpContext *>(context)->session;
+        session->destroyClipboard();
     }
 }
 
@@ -338,6 +351,21 @@ bool RdpSession::start()
         settings->DesktopHeight = m_size.height();
     }
 
+    switch (m_preferences->colorDepth()) {
+    case RdpHostPreferences::ColorDepth::Auto:
+    case RdpHostPreferences::ColorDepth::Depth32:
+        settings->ColorDepth = 32;
+        break;
+    case RdpHostPreferences::ColorDepth::Depth24:
+        settings->ColorDepth = 24;
+        break;
+    case RdpHostPreferences::ColorDepth::Depth16:
+        settings->ColorDepth = 16;
+        break;
+    case RdpHostPreferences::ColorDepth::Depth8:
+        settings->ColorDepth = 8;
+    }
+
     switch (m_preferences->acceleration()) {
     case RdpHostPreferences::Acceleration::ForceGraphicsPipeline:
         settings->SupportGraphicsPipeline = true;
@@ -372,21 +400,6 @@ bool RdpSession::start()
         break;
     }
 
-    switch (m_preferences->colorDepth()) {
-    case RdpHostPreferences::ColorDepth::Auto:
-    case RdpHostPreferences::ColorDepth::Depth32:
-        settings->ColorDepth = 32;
-        break;
-    case RdpHostPreferences::ColorDepth::Depth24:
-        settings->ColorDepth = 24;
-        break;
-    case RdpHostPreferences::ColorDepth::Depth16:
-        settings->ColorDepth = 16;
-        break;
-    case RdpHostPreferences::ColorDepth::Depth8:
-        settings->ColorDepth = 8;
-    }
-
     settings->FastPathOutput = true;
     settings->FastPathInput = true;
     settings->FrameMarkerCommandEnabled = true;
@@ -535,7 +548,7 @@ bool RdpSession::sendEvent(QEvent *event, QWidget *source)
     case QEvent::MouseButtonDblClick:
     case QEvent::MouseMove: {
         auto mouseEvent = static_cast<QMouseEvent *>(event);
-        auto position = mouseEvent->localPos();
+        auto position = mouseEvent->position();
         auto sourceSize = QSizeF{source->size()};
 
         auto x = (position.x() / sourceSize.width()) * m_size.width();
@@ -657,9 +670,9 @@ bool RdpSession::onPostConnect()
 
     auto settings = m_freerdp->settings;
 
-    m_videoBuffer = QImage(settings->DesktopWidth, settings->DesktopHeight, QImage::Format_RGBA8888);
+    m_videoBuffer = QImage(settings->DesktopWidth, settings->DesktopHeight, QImage::Format_RGBX8888);
 
-    if (!gdi_init_ex(m_freerdp, PIXEL_FORMAT_RGBA32, m_videoBuffer.bytesPerLine(), m_videoBuffer.bits(), nullptr)) {
+    if (!gdi_init_ex(m_freerdp, PIXEL_FORMAT_RGBX32, m_videoBuffer.bytesPerLine(), m_videoBuffer.bits(), nullptr)) {
         qCWarning(KRDC) << "Could not initialize GDI subsystem";
         return false;
     }
@@ -674,6 +687,7 @@ bool RdpSession::onPostConnect()
 
     m_freerdp->update->EndPaint = endPaint;
     m_freerdp->update->DesktopResize = resizeDisplay;
+    m_freerdp->update->PlaySound = playSound;
 
     freerdp_keyboard_init_ex(settings->KeyboardLayout, settings->KeyboardRemappingList);
 
@@ -809,6 +823,12 @@ bool RdpSession::onResizeDisplay()
     return true;
 }
 
+bool RdpSession::onPlaySound()
+{
+    QApplication::beep();
+    return true;
+}
+
 void RdpSession::run()
 {
     auto rdpC = reinterpret_cast<rdpContext *>(m_context);
@@ -860,3 +880,27 @@ void RdpSession::emitErrorMessage()
 
     Q_EMIT errorMessage(error);
 }
+
+void RdpSession::initializeClipboard(RdpContext *rdpC, CliprdrClientContext *cliprdr)
+{
+    if (!rdpC || !cliprdr) {
+        return;
+    }
+    m_clipboard = std::make_unique<RdpClipboard>(rdpC, cliprdr);
+}
+
+void RdpSession::destroyClipboard()
+{
+    if (m_clipboard) {
+        m_clipboard.reset(nullptr);
+    }
+}
+
+bool RdpSession::sendClipboard(const QMimeData *data)
+{
+    if (!m_clipboard) {
+        return false;
+    }
+
+    return m_clipboard->sendClipboard(data);
+}
