@@ -48,8 +48,6 @@ RdpView::RdpView(QWidget *parent, const QUrl &url, KConfigGroup configGroup, con
         m_port = TCP_PORT_RDP;
     }
 
-    setMouseTracking(true);
-
     m_hostPreferences = std::make_unique<RdpHostPreferences>(configGroup);
 }
 
@@ -86,7 +84,7 @@ QSize RdpView::sizeHint() const
         return m_session->size().scaled(parentWidget()->size(), Qt::KeepAspectRatio);
     }
 
-    return m_session->size();
+    return m_session->size() / devicePixelRatio();
 }
 
 void RdpView::startQuitting()
@@ -95,6 +93,8 @@ void RdpView::startQuitting()
         return; // ignore repeated triggers
     }
 
+    unpressModifiers();
+
     qCDebug(KRDC) << "Stopping RDP session";
     m_quitting = true;
     m_session->stop();
@@ -174,6 +174,9 @@ void RdpView::handleError(const unsigned int error)
     case FREERDP_ERROR_CONNECT_WRONG_PASSWORD:
         title = i18nc("@title:dialog", "Login Failure");
         message = i18nc("@label", "Unable to login with the provided credentials. Please double check the user and password.");
+        if (m_password.isEmpty()) {
+            deleteWalletPassword();
+        }
         break;
     case FREERDP_ERROR_CONNECT_ACCOUNT_LOCKED_OUT:
     case FREERDP_ERROR_CONNECT_ACCOUNT_EXPIRED:
@@ -318,63 +321,29 @@ void RdpView::paintEvent(QPaintEvent *event)
     painter.setClipRect(event->rect());
 
     auto image = *m_session->videoBuffer();
+    image.setDevicePixelRatio(devicePixelRatio());
 
     if (m_hostPreferences->scaleToSize()) {
-        painter.drawImage(QPoint{0, 0}, image.scaled(size(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        painter.drawImage(QPoint{0, 0}, image.scaled(size() * devicePixelRatio(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
     } else {
         painter.drawImage(QPoint{0, 0}, image);
     }
     painter.end();
 }
 
-void RdpView::keyPressEvent(QKeyEvent *event)
+void RdpView::handleKeyEvent(QKeyEvent *event)
 {
     m_session->sendEvent(event, this);
-    event->accept();
 }
 
-void RdpView::keyReleaseEvent(QKeyEvent *event)
+void RdpView::handleMouseEvent(QMouseEvent *event)
 {
     m_session->sendEvent(event, this);
-    event->accept();
 }
 
-void RdpView::mousePressEvent(QMouseEvent *event)
-{
-    if (!hasFocus()) {
-        setFocus();
-    }
-
-    m_session->sendEvent(event, this);
-    event->accept();
-}
-
-void RdpView::mouseDoubleClickEvent(QMouseEvent *event)
-{
-    if (!hasFocus()) {
-        setFocus();
-    }
-
-    m_session->sendEvent(event, this);
-    event->accept();
-}
-
-void RdpView::mouseReleaseEvent(QMouseEvent *event)
+void RdpView::handleWheelEvent(QWheelEvent *event)
 {
     m_session->sendEvent(event, this);
-    event->accept();
-}
-
-void RdpView::mouseMoveEvent(QMouseEvent *event)
-{
-    m_session->sendEvent(event, this);
-    event->accept();
-}
-
-void RdpView::wheelEvent(QWheelEvent *event)
-{
-    m_session->sendEvent(event, this);
-    event->accept();
 }
 
 void RdpView::onRectangleUpdated(const QRect &rect)
@@ -382,3 +351,8 @@ void RdpView::onRectangleUpdated(const QRect &rect)
     m_pendingRectangle = rect;
     update();
 }
+
+void RdpView::handleLocalClipboardChanged(const QMimeData *data)
+{
+    m_session->sendClipboard(data);
+}
\ No newline at end of file
